/*---------------------------------------------------------------------------
 * Copyright (C) 2007 Maxim Integrated Products, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated Products
 * shall not be used except as stated in the Maxim Integrated Products
 * Branding Policy.
 *---------------------------------------------------------------------------
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32; // For System Registry classes

namespace SetPrt64
{
   public partial class FormSetPrt64 : Form
   {
      public FormSetPrt64()
      {
         InitializeComponent();
      }

      /// <summary> Returns the TMEX x86 or x64 DefaultPortNum as an integer from the 
      /// system registry under the following key:
      /// HKEY_CURRENT_USER\Software\Maxim Integrated Products\1-Wire Drivers
      /// </summary>
      /// <returns>  <code>DefaultPortNum</code> This can be an integer from 0-15
      /// </returns>
      private int getPortNumFromRegistry()
      {
         string portNum;
         int returnVal = -1;
         try
         {
            RegistryKey MaximKey = Registry.CurrentUser; // Create MaximKey set to HKEY_CURRENT_USER
            MaximKey = MaximKey.OpenSubKey("Software\\Maxim Integrated Products\\1-Wire Drivers"); // Set it to HKEY_CURRENT_USER\Maxim Integrated Products\1-Wire Drivers
            portNum = MaximKey.GetValue("DefaultPortNum").ToString();
            MaximKey.Close();
            portNum = portNum.Trim();
            returnVal = Convert.ToInt32(portNum);
         }
         catch(Exception err)
         {
            toolStripStatusLabel.Text = "Status:  " + err.ToString(); 
         }
         return returnVal;
      }

      /// <summary> Returns the TMEX x86 or x64 DefaultPortType as an integer from the 
      /// system registry under the following key:
      /// HKEY_CURRENT_USER\Software\Maxim Integrated Products\1-Wire Drivers
      /// </summary>
      /// <returns>  <code>DefaultPortType</code> This can be an integer with 
      /// typical values of 1 (legacy serial), 2 (parallel), 5 (serial), 6 (USB)
      /// </returns>
      private int getPortTypeFromRegistry()
      {
         string portType;
         int returnVal = -1;
         try
         {
            RegistryKey MaximKey = Registry.CurrentUser; // Create MaximKey set to HKEY_CURRENT_USER
            MaximKey = MaximKey.OpenSubKey("Software\\Maxim Integrated Products\\1-Wire Drivers"); // Set it to HKEY_CURRENT_USER\Maxim Integrated Products\1-Wire Drivers
            portType = MaximKey.GetValue("DefaultPortType").ToString();
            MaximKey.Close();
            portType = portType.Trim();
            returnVal = Convert.ToInt32(portType);
         }
         catch (Exception err)
         {
            toolStripStatusLabel.Text = "Status:  " + err.ToString();
         }
         return returnVal;
      }

      /// <summary> Retrieve DefaultPortNum and DefaultPortType from the 
      /// system registry and updates the GUI components      
      /// </summary>
      /// <returns>  <code>true</code> if successful,
      /// <code>false</code> otherwise.
      /// </returns>
      private bool getDefaultsAndSetGUI()
      {
         int portNum = -1;
         int portType = -1;
         bool isValidPortType = false;
         bool isValidPortNum = false;
         string portTypeString = "";

         try
         {
            portNum = getPortNumFromRegistry();
            portType = getPortTypeFromRegistry();
            if ((portType == 1) || (portType == 2) || (portType == 5) || (portType == 6))
            {
               isValidPortType = true;
            }
            else
            {
               toolStripStatusLabel.Text = "ERROR!  Default port type missing from registry.";
               return false;
            }
            if ((portNum > -1) && (portNum < 16))
            {
               isValidPortNum = true;
            }
            else
            {
               toolStripStatusLabel.Text = "ERROR!  Default port number missing from registry.";
               return false;
            }
            if (isValidPortNum && isValidPortType)
            {
               // set radio boxes to correct port type
               if (portType == 6)
               {
                  radioButtonUSB.Checked = true;
                  portTypeString = "DS9490 USB";
               }
               if (portType == 5)
               {
                  radioButtonSerial.Checked = true;
                  portTypeString = "DS9097U Serial";
               }
               if (portType == 2)
               {
                  radioButtonParallel.Checked = true;
                  portTypeString = "DS1410E Parallel";
               }
               if (portType == 1)
               {
                  radioButtonLegacy.Checked = true;
                  portTypeString = "DS9097E Legacy Serial";
               }
               // set spin box to correct port number
               numericUpDownPortNum.Value = portNum;
               toolStripStatusLabel.Text = "Defaults:  Port Type = " + portTypeString + "    Port Number = " + portNum;
               return true;
            }
         }
         catch (Exception err)
         {
            toolStripStatusLabel.Text = "Status:  " + err.ToString();
         }
         return false;
      }

      /// <summary> Sets up the GUI form with registry reads to 
      /// retrieve DefaultPortNum and DefaultPortType upon form load.      
      /// </summary>
      /// <returns> nothing.
      /// </returns>
      private void FormSetPrt64_Load(object sender, EventArgs e)
      {
         try
         {
            numericUpDownPortNum.Value = 1;
            toolStripStatusLabel.Text = "Status:  OK";
            getDefaultsAndSetGUI();
         }
         catch(Exception err)
         {
            toolStripStatusLabel.Text = "Status:  " + err.ToString(); 
         }
      }

      /// <summary> When user clicks "buttonRegistryRead" button, registry
      /// data is retrieved and GUI components updated to reflect the results.
      /// </summary>
      /// <returns>nothing.
      /// </returns>
      private void buttonRegistryRead_Click(object sender, EventArgs e)
      {
         getDefaultsAndSetGUI();
      }

      /// <summary> When user clicks "buttonRegistryWrite" button, registry
      /// data is updated for "DefaultPortNum" and "DefaultPortType" based on 
      /// GUI component values that the user selected.
      /// </summary>
      /// <returns>nothing.
      /// </returns>
      private void buttonRegistryWrite_Click(object sender, EventArgs e)
      {
         int newPortType = 6;
         try
         {
            RegistryKey MaximKey = Registry.CurrentUser; // Create MaximKey set to HKEY_CURRENT_USER
            // Open Subkey with write permission
            MaximKey = MaximKey.OpenSubKey("Software\\Maxim Integrated Products\\1-Wire Drivers", true); // Set it to HKEY_CURRENT_USER\Maxim Integrated Products\1-Wire Drivers
            if (radioButtonUSB.Checked == true) newPortType = 6;
            if (radioButtonSerial.Checked == true) newPortType = 5;
            if (radioButtonParallel.Checked == true) newPortType = 2;
            if (radioButtonLegacy.Checked == true) newPortType = 1;
            MaximKey.SetValue("DefaultPortType", newPortType.ToString());
            MaximKey.SetValue("DefaultPortNum", numericUpDownPortNum.Value.ToString());
            MaximKey.Close();
            getDefaultsAndSetGUI();
         }
         catch (Exception err)
         {
            toolStripStatusLabel.Text = "Status:  " + err.ToString();
         }
      }
   }
}